﻿using System.Linq;
using System.Windows;
using Infragistics.Windows.DataPresenter;
using Infragistics.Windows.DataPresenter.Events;
using Infragistics.Windows.Editors;
using System;
using System.Windows.Threading;

namespace GroupingOnDemand
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private readonly MainViewModel _ctx = new MainViewModel();

        public MainWindow()
        {
            InitializeComponent();

            DataContext = _ctx;
            _ctx.PropertyChanged += (s, e) =>
            {
                if (e.PropertyName == "Persons")
                    UpdateGrouping();
            };
            xdg.RecordsInViewChanged += xdg_RecordsInViewChanged;
        }

        private int _recordCount;
        void xdg_RecordsInViewChanged(object sender, RecordsInViewChangedEventArgs e)
        {
            if (_recordCount != xdg.Records.Count)
            {
                _recordCount = xdg.Records.Count;
                UpdateGrouping();
            }
        }

        private void xdg_Loaded(object sender, RoutedEventArgs e)
        {
            //loaded event will be triggered before list has been changed.
            UpdateGrouping();
        }

        private void UpdateGrouping()
        {
            var fieldLayout = xdg.FieldLayouts.FirstOrDefault();
            if (fieldLayout == null) return;

            fieldLayout.SortedFields.Clear();
            var sortDesc = new FieldSortDescription();
            sortDesc.FieldName = "Gender";
            sortDesc.IsGroupBy = true;
            fieldLayout.SortedFields.Add(sortDesc);

            //evaluate groups
            var groupsCount = xdg.RecordManager.Groups.Count;
            if (groupsCount < 2)
                fieldLayout.SortedFields.Clear();
        }

        /// <summary>
        /// http://www.infragistics.com/community/forums/t/7569.aspx
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnValueChanged(object sender, RoutedPropertyChangedEventArgs<object> e)
        {
            ValueEditor editor = (ValueEditor)e.OriginalSource;

            CellValuePresenter cvp = (CellValuePresenter)editor.Host;

            Dispatcher.BeginInvoke(new Action(() =>
            {
                RecordPresenter presenter = DataRecordPresenter.FromRecord(cvp.Record);

                if (presenter != null)
                {
                    //Since the RecordPresenter was found, this ValueChanged event was raised because a change was made to the underlying data model.
                    UpdateGrouping();
                }
            }), DispatcherPriority.Background);
            //If the xamEditor's ValueChanged event is raised because a record is scrolled into view, the RecordPresenter will be null.    

        }

        private void OnLoadItemsClick(object sender, RoutedEventArgs e)
        {
            _ctx.LoadPersons(true);
        }

        private void OnLoadItems2Click(object sender, RoutedEventArgs e)
        {
            _ctx.LoadPersons(false);
        }

        private void OnChangeGenderClick(object sender, RoutedEventArgs e)
        {
            if (_ctx.Persons != null && _ctx.Persons.Count > 0)
            {
                var person = _ctx.Persons.LastOrDefault();
                if (person != null)
                {
                    person.Gender = person.Gender == "Male" ? "Female" : "Male";
                    int recCount = xdg.Records.Count;
                    xdg.BringRecordIntoView(xdg.Records[recCount - 1]);
                }
            }
        }

    }
}